/*  main_xt.c

    Main file for  kvis  (X11 image/movie display tool for Karma).

    Copyright (C) 2002-2004  Richard Gooch

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Richard Gooch may be reached by email at  karma-request@atnf.csiro.au
    The postal address is:
      Richard Gooch, c/o ATNF, P. O. Box 76, Epping, N.S.W., 2121, Australia.
*/

/*
    This Karma module will enable on-screen imaging of multiple arrayfiles of
    the general data structure format. The arrayfiles must contain a 2 or 3
    dimensional "Intelligent Array".
    This module runs on an X11 server.


    Written by      Richard Gooch   23-NOV-2002: Copied from kview/main_xt.c

    Updated by      Richard Gooch   30-NOV-2002: Load files specified on
  command line.

    Updated by      Richard Gooch   24-DEC-2002: Made use of new support for
  families of file browsers.

    Updated by      Richard Gooch   28-JAN-2003: No longer initialise visible
  canvas.

    Updated by      Richard Gooch   14-FEB-2003: Set geometry resources for
  main and second display window and first data browser.

    Updated by      Richard Gooch   19-FEB-2003: Added undocumented option
  -cmap_24only.

    Updated by      Richard Gooch   18-MAR-2003: Call XFlush before loading
  data, so that window appears.

    Updated by      Richard Gooch   22-APR-2003: Replaced -cmap_24only option
  with -cmap_noflash.

    Updated by      Richard Gooch   6-MAR-2004: Strip trailing '/' on filenames

    Updated by      Richard Gooch   25-APR-2004: Added -MultiBeam option.

    Updated by      Richard Gooch   7-JUL-2004: Added -no_initial_data_browser
  option.

    Updated by      Richard Gooch   11-JUL-2004: Added -new_window option.

    Last updated by Richard Gooch   5-AUG-2004: Start count from "FILE0" for
  files given on command line.


*/
#include <stdio.h>
#include <stdlib.h>
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Label.h>
#include <karma.h>
#include <k_version.h>
#define NEW_WIN_SCALE
#define X11
#include <k_event_codes.h>
#include <karma_foreign.h>
#include <karma_xtmisc.h>
#include <karma_dsxfr.h>
#include <karma_gls.h>
#include <karma_ext.h>
#include <karma_ds.h>
#include <karma_im.h>
#include <karma_ic.h>
#include <karma_m.h>
#include <Xkw/ImageDisplay.h>
#include <Xkw/Filewin.h>


#define VERSION "1.0.2"


/*  Private functions  */


/*  Private data  */
String fallback_resources[] =
{
    "Kvis.geometry:                                   +0+0",
    "Kvis*pseudoColourCanvas*background:              black",
    "Kvis*directColourCanvas*background:              black",
    "Kvis*trueColourCanvas*background:                black",
    "Kvis*pseudoColourStereoCanvas*background:        black",
    "Kvis*directColourStereoCanvas*background:        black",
    "Kvis*trueColourStereoCanvas*background:          black",
    "Kvis*AnimateControl*positionCanvas.background:   black",
    "Kvis*Canvas.background:                          black",
    "Kvis*pseudoColourCanvas*foreground:              white",
    "Kvis*directColourCanvas*foreground:              white",
    "Kvis*Command*background:                         grey70",
    "Kvis*Repeater*background:                        grey70",
    "Kvis*Ktoggle*background:                         grey80",
    "Kvis*closeButton*background:                     grey90",
    "Kvis*destroyButton*background:                   yellow",
    "Kvis*ChoiceMenu.background:                      turquoise",
    "Kvis*ExclusiveMenu.background:                   turquoise",
    "Kvis*Value*background:                           #d0a0a0",
    "Kvis*ImageDisplay*quit*background:               orange",
    "Kvis*ScrollableList.highlightPixel:              aquamarine3",
    "Kvis*SimpleSlider.foreground:                    Sea Green",
    "Kvis*SimpleProgressMeter.background:             turquoise3",
    "Kvis*SimpleProgressMeter.foreground:             grey60",
    "Kvis*firstDataBrowserShell.geometry:             +540-0",
    "Kvis*imageDisplayShell1.geometry:                -0+0",
    "Kvis*MultiCanvas*Canvas.showBindings:            True",
    "Kvis*ImageDisplay*trackLabel0*font:              8x13bold",
    "Kvis*ImageDisplay*trackLabel1*font:              8x13bold",
    "Kvis*ImageDisplay*trackLabel2*font:              8x13bold",
    "Kvis*ImageDisplay*trackLabel3*font:              8x13bold",
    "Kvis*SimpleMenu*font:                            10x20",
    "Kvis*ImageDisplay.Command.font:                  7x13bold",
    "Kvis*ImageDisplay.applicationBox.Command.font:   7x13bold",
    "Kvis*ImageDisplay.Ktoggle.font:                  7x13bold",
    "Kvis*ImageDisplay.ChoiceMenu.font:               7x13bold",
    "Kvis*ImageDisplay.exportMenu.menuButton.font:    7x13bold",
    "Kvis*ImageDisplay.overlayMenu.menuButton.font:   7x13bold",
    "Kvis*ImageDisplay*zoomMenu*Unzoom*foreground:    red",
    "Kvis*ImageDisplay.numTrackLabels:                3",
    "Kvis*SimpleSlider.borderWidth:                   0",
    "Kvis*IntensityPolicy*showIntensityReset:         True",
    "Kvis*IntensityPolicy*resetIntensityToggle.state: True",
    "Kvis*IntensityPolicy*autoIntensityScale:         False",
    "Kvis*ZoomPolicy*showMiddleZoom:                  True",
    "Kvis*ZoomPolicy*middleZoomToggle.state:          True",
    "Kvis*ZoomPolicy*showRightZoom:                   True",
    "Kvis*ZoomPolicy*rightZoomToggle.state:           True",
    "Kvis*Filewin.dirname.textSink.font:              6x13",
    "Kvis*arrayForm*sourceLabel.font:                 6x13",
    "Kvis*arrayForm*unitsLabel.font:                  8x13bold",
    "Kvis*overlayForm*sourceLabel.font:               6x13",
    "Kvis*font:                                       9x15bold",
    "Kvis*borderColor:                                black",
    "Kvis*background:                                 aquamarine2",
    "Kvis*foreground:                                 black",
    NULL
};
static XrmOptionDescRec Options[] =
{
    {"-private_cmap", ".topForm.multiCanvas.pseudoColourCanvas.forceNewCmap",
     XrmoptionNoArg, (XPointer) "True"},
    {"-num_colours", "*topForm.cmapSize", XrmoptionSepArg, (XPointer) NULL},
    {"-cmap_master", ".topForm.cmapMaster", XrmoptionSepArg, (XPointer) NULL},
/*    {"-cmap_24only", ".topForm.allowedMask", XrmoptionNoArg, "254"},*/
    {"-fullscreen", ".topForm.fullscreen", XrmoptionNoArg, (XPointer) "True"},
    {"-create_at_init", "*createAtInit", XrmoptionNoArg, (XPointer) "True"},
    {"-no_initial_data_browser", "*popupFirstDataBrowser", XrmoptionNoArg,
     (XPointer) "False"},
    {"-MultiBeam", "*ImageDisplay.numTrackLabels", XrmoptionNoArg,
     (XPointer) "4"},
    {"-verbose", "*verbose", XrmoptionNoArg, (XPointer) "True"},
};
static Widget main_shell = NULL;
static Widget image_display = NULL;


int main (int argc, char **argv)
{
    flag avoid_flash;
    flag do_multibeam = FALSE;
    int argcount;
    XtAppContext app_context;
    Widget filewin, filepopup;
    char title_name[STRING_LENGTH];
    /*static char function_name[] = "main";*/

    /*  Initialise module  */
    im_register_module_version_date (VERSION);
    im_register_lib_version (KARMA_VERSION);
    /*  Early processing of command-line arguments before dual-use ones get
	stripped out  */
    for (argcount = 1; argcount < argc; ++argcount)
    {
	if (strcmp (argv[argcount], "-MultiBeam") == 0) do_multibeam = TRUE;
    }
    /*  Start up Xt  */
    main_shell = xtmisc_init_app_initialise (&app_context, "Kvis",
					     Options, XtNumber (Options),
					     &argc, argv, fallback_resources,
					     XTMISC_INIT_ATT_MIN_CCELLS, 100,
					     XTMISC_INIT_ATT_NUM_CCELLS, 200,
					     XTMISC_INIT_ATT_AVOID_FLASH,
					         &avoid_flash,
					     XTMISC_INIT_ATT_COMMS_SETUP, TRUE,
					     XTMISC_INIT_ATT_BECOME_SERVER,
					         TRUE,
					     XTMISC_INIT_ATT_TITLE, title_name,
					     XTMISC_INIT_ATT_END,
					     NULL);
    dsxfr_register_connection_limits (1, -1);
    xtmisc_set_icon (main_shell, ic_write_kimage_icon);
    if (do_multibeam)
    {
	if ( !gls_test_running () )
	{
	    fputs ("Glish not available, can't run in MultiBeam mode\n",
		   stderr);
	    exit (RV_UNDEF_ERROR);
	}
	if ( !ext_add_service ("ImageDisplay", "kvis-multibeam", NULL) )
	{
	    fputs ("Failed to load scroll buffer plugin\n", stderr);
	    exit (RV_UNDEF_ERROR);
	}
    }
    if (avoid_flash)
	image_display = XtVaCreateManagedWidget ("topForm",
						 imageDisplayWidgetClass,
						 main_shell,
						 XtNborderWidth, 0,
						 XkwNviewDatasets, TRUE,
						 XkwNnumDatasets, 0,
						 "allowedMask", 254,
						 NULL);
    else image_display = XtVaCreateManagedWidget ("topForm",
						  imageDisplayWidgetClass,
						  main_shell,
						  XtNborderWidth, 0,
						  XkwNviewDatasets, TRUE,
						  XkwNnumDatasets, 0,
						  NULL);
    filepopup = XtNameToWidget (image_display, "filewinPopup");
    XtVaGetValues (filepopup, XkwNfilewinWidget, &filewin, NULL);
    XtVaSetValues (filepopup,
		   XkwNfilenameTester, XkwFilewinStandardFileTester_nD,
		   NULL);
    XtVaSetValues (filewin,
		   XkwNforwardSyntheticEvents, True,
		   XkwNtrapDirectoryDatasets, True,
		   NULL);
    XtRealizeWidget (main_shell);
    XFlush ( XtDisplay (main_shell) );  /*  Want window to appear now  */
    xtmisc_init_post_realise_arg_process (&argc, argv);
    /*  Option processing after widgets created and realised  */
    for (argcount = 1; argcount < argc; ++argcount)
    {
	size_t len;
	multi_array *multi_desc;
	char txt[STRING_LENGTH], fname[STRING_LENGTH];

	len = strlen (argv[argcount]);
	if (len + 10 > STRING_LENGTH) continue;
	sprintf (txt, "FILE%d", argcount - 1);
	if (strcmp (argv[argcount], "-new_window") == 0)
	{
	    unsigned int window_type;
	    Widget w;

	    ++argcount;
	    if (strcmp (argv[argcount], "slave_cmap") == 0)
		window_type = XkwNEW_WINDOW_SLAVE_CMAP;
	    else if (strcmp (argv[argcount], "independent_cmap") == 0)
		window_type = XkwNEW_WINDOW_INDEPENDENT_CMAP;
	    else
	    {
		fprintf (stderr, "Invalid window type: \"%s\"\n",
			 argv[argcount]);
		continue;
	    }
	    w = XkwImageDisplayCreateWindow (image_display, window_type);
	    XtPopup (w, XtGrabNone);
	    continue;
	}
	if (strncmp (argv[argcount], "./", 2) == 0)
	{
	    strcpy (fname, argv[argcount] + 2);
	    len -= 2;
	}
	else strcpy (fname, argv[argcount]);
	if (fname[len - 1] == '/') fname[--len] = '\0';
	if ( ( multi_desc =
	       foreign_guess_and_read (fname, K_CH_MAP_LOCAL,
				       FALSE, NULL,
				       FA_GUESS_READ_END) )
	     == NULL )
	{
	    fprintf (stderr, "Error reading file: \"%s\"\n", fname);
	    continue;
	}
	ds_event_dispatch (multi_desc, txt, fname);
	ds_dealloc_multi (multi_desc);
	m_clear (fname, STRING_LENGTH);  /*  Force me to take care  */
    }
    XtAppMainLoop (app_context);
    return (RV_OK);
}   /*  End Function main  */


/*  Private functions follow  */

#ifdef OS_SunOS

static void dummy ()
/*  [PURPOSE] This routine is needed for SunOS, because of a problem with
    shared archive use. The <<_XtToolkitInitialize>> symbol is defined in
    the Xt shared archive, but it appears the shared archive is not linked for
    this application. Other applications (like <<krenzo>>) have no such
    problems. This is a truly bizzare bug.
*/
{
    XtToolkitInitialize ();
}   /*  End Function dummy  */
#endif
