/*LINTLIBRARY*/
/*  misc.c

    This code provides miscellaneous routines for pixel canvases.

    Copyright (C) 2002  Richard Gooch

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Richard Gooch may be reached by email at  karma-request@atnf.csiro.au
    The postal address is:
      Richard Gooch, c/o ATNF, P. O. Box 76, Epping, N.S.W., 2121, Australia.
*/

/*  This file contains miscellaneous routines for pixel canvases.


    Written by      Richard Gooch   31-JAN-2002

    Last updated by Richard Gooch   31-JAN-2002


*/
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <karma_kwin.h>


/*  Public functions follow  */

/*EXPERIMENTAL_FUNCTION*/
flag kwin_get_colour_harder (KPixCanvas canvas, CONST char *colourname,
			     unsigned long *pixel_value)
/*  [SUMMARY] Get (possibly allocate) a colourcell for a canvas.
    [PURPOSE] This function will get a colourcell for a canvas. It is similar
    to the [<kwin_get_colour>] function, except that tries harder for
    TrueColour and DirectColour visuals, by using the pixel masks.
    <canvas> The canvas.
    <colourname> The name of the colour to get.
    <pixel_value> The pixel value will be written here.
    [NOTE] The pixel value is valid ONLY for this canvas. ALSO: the pixel value
    becomes invalid EVERY time the canvas is refreshed/ resized. If the canvas
    is refreshed/ resized, this routine MUST be called again.
    [RETURNS] TRUE if the colourcell was allocated, else FALSE.
*/
{
    unsigned int visual;
    unsigned long red, green, blue;

    kwin_get_attributes (canvas,
			 KWIN_ATT_VISUAL, &visual,
			 KWIN_ATT_END);
    if ( (visual != KWIN_VISUAL_DIRECTCOLOUR) &&
	 (visual != KWIN_VISUAL_TRUECOLOUR) )
	return kwin_get_colour (canvas, colourname, pixel_value,
				NULL, NULL, NULL);
    kwin_get_attributes (canvas,
			 KWIN_ATT_PIX_RED_MASK, &red,
			 KWIN_ATT_PIX_GREEN_MASK, &green,
			 KWIN_ATT_PIX_BLUE_MASK, &blue,
			 KWIN_ATT_END);
    if (strcmp (colourname, "black") == 0) *pixel_value = 0;
    else if (strcmp (colourname, "white") == 0) *pixel_value = red |green|blue;
    else if (strcmp (colourname, "red") == 0) *pixel_value = red;
    else if (strcmp (colourname, "green") == 0) *pixel_value = green;
    else if (strcmp (colourname, "blue") == 0) *pixel_value = blue;
    else if (strcmp (colourname, "yellow") == 0) *pixel_value = red | green;
    else if (strcmp (colourname, "magenta") == 0) *pixel_value = red | blue;
    else if (strcmp (colourname, "cyan") == 0) *pixel_value = green | blue;
    else return kwin_get_colour (canvas, colourname, pixel_value,
				 NULL, NULL, NULL);
    return (TRUE);
}   /*  End Function kwin_get_colour_harder  */
