/*LINTLIBRARY*/
/*  MakeDataBrowser.c

    This code provides a browser for making data for Xt.

    Copyright (C) 2004  Richard Gooch

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Richard Gooch may be reached by email at  karma-request@atnf.csiro.au
    The postal address is:
      Richard Gooch, c/o ATNF, P. O. Box 76, Epping, N.S.W., 2121, Australia.
*/

/*  This file contains all routines needed for manipulating a popup make data
    browser widget for Xt.


    Written by      Richard Gooch   16-MAR-2004

    Last updated by Richard Gooch   31-MAR-2004: Increased width.


*/
#include <unistd.h>
#include <stdio.h>
#include <Xkw/MakeDataBrowserP.h>
#include <X11/Xos.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Command.h>    
#include <X11/Xaw/Label.h>    
#include <X11/Xaw/Form.h>
#include <karma.h>
#include <k_event_codes.h>
#include <karma_construct.h>
#include <karma_listw.h>
#include <karma_xtmisc.h>
#include <Xkw/ScrollableList.h>
#include <Xkw/ExclusiveMenu.h>
#include <Xkw/SimpleTextEntry.h>


/*  Private functions  */
STATIC_FUNCTION (void _MakeDataBrowser_Initialise,
		 (Widget request, Widget new) );
STATIC_FUNCTION (void _MakeDataBrowser_Destroy, (Widget W) );
STATIC_FUNCTION (void _MakeDataBrowser_algorithm_cbk,
		 (Widget w, XtPointer client_data, XtPointer call_data) );
STATIC_FUNCTION (void _MakeDataBrowser_change_name,
		 (void *info, CONST char *name) );
STATIC_FUNCTION (void _MakeDataBrowser_make_cbk,
		 (Widget w, XtPointer client_data, XtPointer call_data) );


#define offset(field) XtOffset(MakeDataBrowserWidget, makeDataBrowser.field)

static XtResource MakeDataBrowserResources[] = 
{
    {"empty", "Empty", XtRInt, sizeof (int), offset (dummy), XtRImmediate, 0},
};

#undef offset


MakeDataBrowserClassRec makeDataBrowserClassRec = 
{
  {        /* CoreClassPart */
    (WidgetClass)&topLevelShellClassRec,      /* superclass */
    "MakeDataBrowser",                        /* class_name */
    sizeof(MakeDataBrowserRec),               /* widget_size */
    NULL,                                     /* class_initialise */
    NULL,                                     /* class_part_initialise */
    FALSE,                                    /* class_init */
    (XtInitProc)_MakeDataBrowser_Initialise,  /* initialise */
    NULL,                                     /* initialise_hook */
    XtInheritRealize,                         /* realize */
    NULL,                                     /* actions */
    0,                                        /* num_actions */
    MakeDataBrowserResources,                 /* resources */
    XtNumber(MakeDataBrowserResources),       /* num_resources */
    NULLQUARK,                                /* xrm_class */
    TRUE,                                     /* compress_motion */
    TRUE,                                     /* compress_exposure */
    TRUE,                                     /* compress_enterleave */
    TRUE,                                     /* visible_interest */
    _MakeDataBrowser_Destroy,                 /* destroy */
    XtInheritResize,                          /* resize */
    NULL,                                     /* expose */
    NULL,                                     /* set_values */
    NULL,                                     /* set_values_hook */
    XtInheritSetValuesAlmost,                 /* set_values_almost */
    NULL,                                     /* get_values_hook */
    NULL,                                     /* accept_focus */
    XtVersion,                                /* version */
    NULL,                                     /* callback_private */
    NULL,                                     /* tm_translations */
    NULL,                                     /* query_geometry  */
    NULL,
    NULL,
  },
  {     /* CompositeClassPart */
    /* geometry_manager	  */	XtInheritGeometryManager,
    /* change_managed	  */	XtInheritChangeManaged,
    /* insert_child	  */	XtInheritInsertChild,
    /* delete_child	  */	XtInheritDeleteChild,
    /* extension	  */	NULL
  },{ /* Shell */
    /* extension	  */	NULL
  },{ /* WMShell */
    /* extension	  */	NULL
  },{ /* VendorShell */
    /* extension	  */	NULL
  },{ /* TopLevelShell */
    /* extension	  */	NULL
  },
  {  /* MakeDataBrowserClassPart */
    0 /* empty */
  }
};

WidgetClass makeDataBrowserWidgetClass = (WidgetClass)&makeDataBrowserClassRec;


static void _MakeDataBrowser_Initialise (Widget Request, Widget New)
{
    KListWidget list;
    /*MakeDataBrowserWidget request = (MakeDataBrowserWidget) Request;*/
    MakeDataBrowserWidget new = (MakeDataBrowserWidget) New;
    Widget form, w, upper_widget;
    /*static char function_name[] = "MakeDataBrowser::Initialise";*/

    form = XtVaCreateManagedWidget ("form", formWidgetClass, New,
				    XtNborderWidth, 0,
				    NULL);
    w = XtVaCreateManagedWidget ("closeButton", commandWidgetClass, form,
				 XtNlabel, "Close",
				 NULL);
    XtAddCallback (w, XtNcallback, xtmisc_popdown_cbk, New);
    w = XtVaCreateManagedWidget ("algorithmMenu", exclusiveMenuWidgetClass,
				 form,
				 XtNmenuName, "algorithmMenu",
				 XkwNchoiceName, "Algorithm",
				 XtNfromHoriz, w,
				 XtNtop, XtChainTop,
				 XtNbottom, XtChainTop,
				 XkwNnumItems, KCONSTRUCTARRAY_NUM_ALGORITHMS,
				 XkwNitemStrings,
				     construct_array_algorithm_choices,
				 NULL);
    upper_widget = w;
    XtAddCallback (w, XkwNselectCallback, _MakeDataBrowser_algorithm_cbk, new);
    w = XtVaCreateManagedWidget ("list", scrollableListWidgetClass, form,
				 XtNhorizDistance, 0,
				 XtNwidth, 300,
				 XtNheight, 150,
				 XtNfromVert, upper_widget,
				 XtNtop, XtChainTop,
				 XtNbottom, XtChainBottom,
				 NULL);
    upper_widget = w;
    XtVaGetValues (w, XkwNlist, &list, NULL);
    new->makeDataBrowser.manager =
	construct_array_create_manager (list, _MakeDataBrowser_change_name,
					new);
    construct_array_select_algorithm (new->makeDataBrowser.manager, 0);
    w = XtVaCreateManagedWidget ("nameLabel", labelWidgetClass, form,
				 XtNborderWidth, 0,
				 XtNhorizDistance, 0,
				 XtNfromVert, upper_widget,
				 XtNlabel, "Name:",
				 NULL);
    upper_widget = w;
    w = XtVaCreateManagedWidget ("nameText", simpleTextEntryWidgetClass, form,
				 XtNhorizDistance, 0,
				 XtNfromVert, upper_widget,
				 XtNwidth, 300,
				 XtNeditType, XawtextEdit,
				 NULL);
    upper_widget = w;
    new->makeDataBrowser.name_txt = w;
    XtAddCallback (w, XkwNnewLineCallback, _MakeDataBrowser_make_cbk, New);
    w = XtVaCreateManagedWidget ("makeButton", commandWidgetClass, form,
				 XtNlabel, "Make",
				 XtNfromVert, upper_widget,
				 XtNtop, XtChainBottom,
				 XtNbottom, XtChainBottom,
				 NULL);
    XtAddCallback (w, XtNcallback, _MakeDataBrowser_make_cbk, New);
}   /*  End Function Initialise  */

static void _MakeDataBrowser_Destroy (Widget W)
{
    MakeDataBrowserWidget w = (MakeDataBrowserWidget) W;

    construct_array_destroy_manager (w->makeDataBrowser.manager);
}   /*  End Function Destroy  */

static void _MakeDataBrowser_algorithm_cbk (Widget w, XtPointer client_data,
					    XtPointer call_data)
{
    int index = *(int *) call_data;
    MakeDataBrowserWidget top = client_data;

    construct_array_select_algorithm (top->makeDataBrowser.manager, index);
}   /*  End Function algorithm_cbk   */

static void _MakeDataBrowser_change_name (void *info, CONST char *name)
{
    MakeDataBrowserWidget top = info;

    XtVaSetValues (top->makeDataBrowser.name_txt, XtNstring, name, NULL);
}   /*  End Function change_name  */

static void _MakeDataBrowser_make_cbk (Widget w, XtPointer client_data,
				       XtPointer call_data)
{
    MakeDataBrowserWidget top = client_data;
    String name;

    XtVaGetValues (top->makeDataBrowser.name_txt, XtNstring, &name, NULL);
    construct_array_create_data (top->makeDataBrowser.manager, name);
}   /*  End Function make_cbk   */


/*  Public functions follow  */

/*EXPERIMENTAL_FUNCTION*/
void XkwMakeDataBrowserPopupAll (Widget parent)
/*  [SUMMARY] Popup a browser for a blinkstate.
    <bs> The blinkstate.
    [RETURNS] Nothing.
*/
{
    static Widget popupshell = NULL;

    if (!popupshell)
	popupshell = XtVaCreatePopupShell ("makeDataBrowserShell",
					   makeDataBrowserWidgetClass, parent,
					   NULL);
    XtPopup (popupshell, XtGrabNone);
}   /*  End Function XkwMakeDataBrowserPopup  */
