/*LINTLIBRARY*/
/*  ScrollableList.c

    This code provides a scrollable list widget for Xt.

    Copyright (C) 2004  Richard Gooch

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Richard Gooch may be reached by email at  karma-request@atnf.csiro.au
    The postal address is:
      Richard Gooch, c/o ATNF, P. O. Box 76, Epping, N.S.W., 2121, Australia.
*/

/*

    This file contains all routines needed for a scrollable list widget for Xt.


    Written by      Richard Gooch   7-MAR-2004

    Updated by      Richard Gooch   11-MAR-2004: Initial tweaking complete.

    Last updated by Richard Gooch   28-JUN-2004: Placate GCC.


*/
#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <math.h>
#include <unistd.h>
#ifndef X11
#  define X11
#endif
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/XawInit.h>
#include <X11/Xaw/Scrollbar.h>
#include <karma.h>
#define NEW_WIN_SCALE
#include <karma_listw.h>
#include <karma_kwin.h>
#include <karma_m.h>
#include <Xkw/ScrollableListP.h>
#include <Xkw/Canvas.h>


STATIC_FUNCTION (void _ScrollableList_Initialise,
		 (Widget request, Widget new) );
STATIC_FUNCTION (Boolean _ScrollableList_SetValues, () );
STATIC_FUNCTION (void _ScrollableList_canvas_realise_cbk,
		 (Widget w, XtPointer client_data, XtPointer call_data) );
STATIC_FUNCTION (void _ScrollableList_att_notify_func,
		 (KListWidget list, void **info) );
STATIC_FUNCTION (void _ScrollableList_scroll_cbk,
		 (Widget w, XtPointer client_data, XtPointer call_data) );
STATIC_FUNCTION (void _ScrollableList_jump_cbk,
		 (Widget w, XtPointer client_data, XtPointer call_data) );


#define offset(field) XtOffsetOf(ScrollableListRec, scrollableList.field)

static XtResource resources[] =
{
    {XkwNlist, XkwCList, XtRPointer, sizeof (XtPointer),
     offset(list), XtRImmediate, NULL},
    {XkwNcanvas, XkwCCanvas, XtRPointer, sizeof (XtPointer),
     offset(canvas), XtRImmediate, NULL},
    {XkwNhighlightPixel, XkwCHighlightPixel, XtRPixel, sizeof (Pixel),
     offset (highlightPixel), XtRString, XtDefaultBackground},
};
#undef offset


ScrollableListClassRec scrollableListClassRec =
{
    {
	/* core fields  */
	(WidgetClass) &formClassRec,    /*  superclass             */
	"ScrollableList",               /*  class_name             */
	sizeof (ScrollableListRec),     /*  widget_size            */
	NULL,                           /*  class_initialise       */
	NULL,                           /*  class_part_initialise  */
	FALSE,                          /*  class_inited           */
	(XtInitProc) _ScrollableList_Initialise, /*  initialise             */
	NULL,                           /*  initialise_hook        */
	XtInheritRealize,               /*  realise                */
	NULL,                           /*  actions                */
	0,                              /*  num_actions            */
	resources,                      /*  resources              */
	XtNumber (resources),           /*  num_resources          */
	NULLQUARK,                      /*  xrm_class              */
	TRUE,                           /*  compress_motion        */
	TRUE,                           /*  compress_exposure      */
	TRUE,                           /*  compress_enterleave    */
	TRUE,                           /*  visible_interest       */
	NULL,                           /*  destroy                */
	XtInheritResize,                /*  resize                 */
	NULL,                           /*  expose                 */
	(XtSetValuesFunc) _ScrollableList_SetValues, /*  set_values  */
	NULL,                           /*  set_values_hook        */
	XtInheritSetValuesAlmost,       /*  set_values_almost      */
	NULL,                           /*  get_values_hook        */
	NULL,                           /*  accept_focus           */
	XtVersion,                      /*  version                */
	NULL,                           /*  callback_private       */
	NULL,                           /*  tm_table               */
	XtInheritQueryGeometry,         /*  query_geometry         */
	XtInheritDisplayAccelerator,    /*  display_accelerator    */
	NULL                            /*  extension              */
    },
    {     /* CompositeClassPart */
	XtInheritGeometryManager        /* geometry_manager        */,
	XtInheritChangeManaged          /* change_managed          */,
	XtInheritInsertChild            /* insert_child            */,
	XtInheritDeleteChild            /* delete_child            */,
	NULL                            /* extension               */
    },
    { /* Constraint */
	NULL                            /* subresourses       */,
	0                               /* subresource_count  */,
	sizeof (ScrollableListConstraintsRec) /* constraint_size    */,
	NULL                            /* initialise         */,
	NULL                            /* destroy            */,
	NULL                            /* set_values         */,
	NULL                            /* extension          */ 
    },
    { /* Form */
	XtInheritLayout                 /* layout             */
    },
    {
	/*  scrollableList fields */
	0                               /*  ignore                 */
    }
};

WidgetClass scrollableListWidgetClass = (WidgetClass) &scrollableListClassRec;


static void _ScrollableList_Initialise (Widget Request, Widget New)
{
    int list_width = 0;
    ScrollableListWidget new = (ScrollableListWidget) New;
    Widget canvas, scrollbar;
    static char function_name[] = "ScrollableListWidget::Initialise";

    new->form.default_spacing = -1;
    if (new->core.width > 0) list_width = new->core.width - 15;
    canvas = XtVaCreateManagedWidget ("listCanvas", canvasWidgetClass, New,
				      XtNborderWidth, 0,
				      XtNwidth, list_width,
				      XtNheight, new->core.height,
				      XtNbackground,new->core.background_pixel,
				      XkwNsilenceUnconsumed, True,
				      XtNtop, XawChainTop,
				      XtNbottom, XawChainBottom,
				      XtNleft, XawChainLeft,
				      XtNright, XawChainRight,
				      XkwNprocessSpecialKeys, False,
				      NULL);
    XtAddCallback (canvas, XkwNrealiseCallback,
		   _ScrollableList_canvas_realise_cbk, New);
    new->scrollableList.canvas_widget = canvas;
    scrollbar = XtVaCreateManagedWidget ("listScrollbar", scrollbarWidgetClass,
					 New,
					 XtNheight, new->core.height,
					 XtNfromHoriz, canvas,
					 XtNhorizDistance, 0,
					 XtNtop, XawChainTop,
					 XtNbottom, XawChainBottom,
					 XtNleft, XawChainRight,
					 XtNright, XawChainRight,
					 NULL);
    new->scrollableList.scrollbar = scrollbar;
    XtAddCallback (scrollbar, XtNscrollProc, _ScrollableList_scroll_cbk, New);
    XtAddCallback (scrollbar, XtNjumpProc, _ScrollableList_jump_cbk, New);
    new->scrollableList.list = listw_create ();
    if (!new->scrollableList.list) m_abort (function_name, "list widget");
    listw_register_att_notify_func (new->scrollableList.list,
				    _ScrollableList_att_notify_func, new);
}   /*  End Function Initialise  */

static Boolean _ScrollableList_SetValues (Widget Current, Widget Request,
					  Widget New)
{
    ScrollableListWidget current = (ScrollableListWidget) Current;
    /*ScrollableListWidget request = (ScrollableListWidget) Request;*/
    ScrollableListWidget new = (ScrollableListWidget) New;
    /*static char function_name[] = "ScrollableListWidget::SetValues";*/

    new->scrollableList.list = current->scrollableList.list;
    new->scrollableList.canvas = current->scrollableList.canvas;
    if (new->scrollableList.highlightPixel !=
	current->scrollableList.highlightPixel)
	listw_set_attributes (new->scrollableList.list, FALSE,
			      LISTW_ATT_HIGHLIGHT_PIXEL,
			          new->scrollableList.highlightPixel,
			      LISTW_ATT_END);
    return (True);
}   /*  End Function SetValues  */

static void _ScrollableList_canvas_realise_cbk (Widget w,
						XtPointer client_data,
						XtPointer call_data)
/*  [SUMMARY] Canvas realise callback.
    [PURPOSE] This routine is called when a Canvas widget is realised.
    <w> The Canvas widget that has just been realised.
    <client_data> The ScrollableList widget.
    <call_data> The pixel canvas(es).
    [RETURNS] Nothing.
*/
{
    unsigned long pixel_value;
    ScrollableListWidget top = client_data;

    top->scrollableList.canvas = call_data;
    listw_bind (top->scrollableList.list, top->scrollableList.canvas);
    XtVaGetValues (top->scrollableList.canvas_widget,
		   XtNforeground, &pixel_value,
		   NULL);
    listw_set_attributes (top->scrollableList.list, FALSE,
			  LISTW_ATT_FOREGROUND_PIXEL, pixel_value,
			  LISTW_ATT_HIGHLIGHT_PIXEL,
			      top->scrollableList.highlightPixel,
			  LISTW_ATT_END);
}   /*  End Function canvas_realise_cbk   */

static void _ScrollableList_att_notify_func (KListWidget list, void **info)
/*  [SUMMARY] Registers a change in the attributes of a list widget.
    <list> The list widget.
    <info> A pointer to an arbitrary information pointer.
    [RETURNS] Nothing.
*/
{
    unsigned int num_strings, top_position;
    float f_fraction, f_position;
    double d_fraction;
    int *i_frac = (int *) &f_fraction;
    int *i_thumb = (int *) &f_position;
    ScrollableListWidget top = *info;

    listw_get_attributes (list,
			  LISTW_ATT_NUM_STRINGS, &num_strings,
			  LISTW_ATT_TOP_POSITION, &top_position,
			  LISTW_ATT_SHOWN_FRACTION, &d_fraction,
			  LISTW_ATT_END);
    f_position = (float) top_position / (float) num_strings;
    f_fraction = d_fraction;
    XtVaSetValues (top->scrollableList.scrollbar,
		   XtNshown, *i_frac,
		   XtNtopOfThumb, *i_thumb,
		   NULL);
}   /*  End Function att_notify_func  */

static void _ScrollableList_scroll_cbk (Widget w, XtPointer client_data,
					XtPointer call_data)
/*  [SUMMARY] Incremental scrolling callback.
    [RETURNS] Nothing.
*/
{
    ScrollableListWidget top = client_data;
    iaddr event_position = (iaddr) call_data;
    Dimension height;
    unsigned int num_strings, top_position;
    double shown_fraction, d_position;

    listw_get_attributes (top->scrollableList.list,
			  LISTW_ATT_NUM_STRINGS, &num_strings,
			  LISTW_ATT_TOP_POSITION, &top_position,
			  LISTW_ATT_SHOWN_FRACTION, &shown_fraction,
			  LISTW_ATT_END);
    XtVaGetValues (w, XtNheight, &height, NULL);
    d_position = (double) top_position / (double) num_strings;
    d_position += (double) event_position / (double) height * shown_fraction;
    if (d_position < 0.0) d_position = 0.0;
    if (d_position + shown_fraction > 1.0) d_position = 1.0 - shown_fraction;
    top_position = (double) num_strings * d_position;
    listw_set_attributes (top->scrollableList.list, TRUE,
			  LISTW_ATT_TOP_POSITION, top_position,
			  LISTW_ATT_END);
}   /*  End Function scroll_cbk   */

static void _ScrollableList_jump_cbk (Widget w, XtPointer client_data,
				      XtPointer call_data)
/*  [SUMMARY] Jump scrolling callback.
    [RETURNS] Nothing.
*/
{
    ScrollableListWidget top = client_data;
    unsigned int num_strings, top_position;
    float f_position = *(float *) call_data;
    double d_position = f_position;
    double shown_fraction;

    listw_get_attributes (top->scrollableList.list,
			  LISTW_ATT_NUM_STRINGS, &num_strings,
			  LISTW_ATT_SHOWN_FRACTION, &shown_fraction,
			  LISTW_ATT_END);
    top_position = (double) num_strings * d_position;
    listw_set_attributes (top->scrollableList.list, TRUE,
			  LISTW_ATT_TOP_POSITION, top_position,
			  LISTW_ATT_END);
}   /*  End Function jump_cbk   */
